@@ -1,5 +1,5 @@
 
-/* GnomeMeeting -- A Video-Conferencing application
+/* Ekiga -- A VoIP and Video-Conferencing application
  * Copyright (C) 2000-2006 Damien Sandras
  *
  * This program is free software; you can redistribute it and/or modify
@@ -13,16 +13,16 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  *
- * GnomeMeting is licensed under the GPL license and as a special exception,
+ * Ekiga is licensed under the GPL license and as a special exception,
  * you have permission to link or otherwise combine this program with the
- * programs OpenH323 and Pwlib, and distribute the combination, without
- * applying the requirements of the GNU GPL to the OpenH323 program, as long
- * as you do follow the requirements of the GNU GPL for all the rest of the
- * software thus combined.
+ * programs OPAL, OpenH323 and PWLIB, and distribute the combination,
+ * without applying the requirements of the GNU GPL to the OPAL, OpenH323
+ * and PWLIB programs, as long as you do follow the requirements of the
+ * GNU GPL for all the rest of the software thus combined.
  */
 
 
@@ -389,10 +389,11 @@ list_from_string (const gchar *str)
 
   g_return_val_if_fail (str != NULL, NULL);
   g_return_val_if_fail (str[0] == '[', NULL);
-  g_return_val_if_fail (str[g_utf8_strlen (str, -1) - 1] == ']', NULL);
+  g_return_val_if_fail (str[strlen (str) - 1] == ']', NULL);
 
   txt = g_strdup (str + 1); /* get the '[' out of the way */
-  txt[g_utf8_strlen (txt, -1) - 1] = 0; /* get the ']' out of the way */
+  txt[strlen (txt) - 1] = 0; /* get the ']' out of the way */
+
 
   if (txt[0] == 0) { /* handle the empty list */
     g_free (txt);
@@ -402,24 +403,32 @@ list_from_string (const gchar *str)
   txt_list = g_strsplit (txt, ",", 0);
   g_free (txt);
   for (tmp_list = txt_list; *tmp_list != NULL; tmp_list++) {
+
     if (item == NULL)
-      item = g_string_new (NULL);
-    if ((*tmp_list)[g_utf8_strlen (*tmp_list, -1) - 1] == '\\') {
-      g_string_append_len (item, *tmp_list, g_utf8_strlen (*tmp_list, -1) - 1);
+      item = g_string_new ("");
+
+    if ((*tmp_list)[strlen (*tmp_list) - 1] == '\\') {
+
+      g_string_append_len (item, *tmp_list,
+			   strlen (*tmp_list) - 1);
       must_concat = TRUE;
-    }
-    else {
-      g_string_append_c (item, ',');
-      g_string_append (item, *tmp_list);
+    } else {
+
+      g_string_append_len (item, *tmp_list,
+			   strlen (*tmp_list));
       must_concat = FALSE;
     }
-    
+
     if (must_concat == FALSE) {
+
       result = g_slist_append (result, item->str);
       g_string_free (item, FALSE);
       item = NULL;
-    }
+
+    } else
+      g_string_append_c (item, ',');
   }
+
   g_strfreev (txt_list);
 
   if (must_concat == TRUE) { /* bad: shouldn't happen */
@@ -428,7 +437,8 @@ list_from_string (const gchar *str)
     g_string_free (item, FALSE);
     item = NULL;
   }
-  
+
+    
   return result;
 }
 
@@ -1170,17 +1180,19 @@ saveconf_timer_callback (gpointer unused)
 void 
 gm_conf_init (int argc, char **argv)
 {
+  gboolean result = FALSE;
   DataBase *db = database_get_default ();
  
-  if (!gm_conf_load_user_conf (db))
-    if (!gm_conf_load_sys_conf (db))
-      g_warning ("Couldn't load system configuration");
+  result = gm_conf_load_sys_conf (db);
+  result = (gm_conf_load_user_conf (db) || result);
+  if (!result)
+    g_warning ("Couldn't load system configuration");
 
   /* those keys aren't found in gnomemeeting's schema */
   gm_conf_set_bool ("/desktop/gnome/interface/menus_have_icons", TRUE);
 
   /* automatic savings */
-  g_timeout_add (120000, (GSourceFunc)saveconf_timer_callback, NULL);
+  g_timeout_add (5000, (GSourceFunc)saveconf_timer_callback, NULL);
 
 }
 
@@ -1237,6 +1249,8 @@ gm_conf_notifier_remove (gpointer identifier)
   g_return_if_fail (identifier != NULL);
 
   g_datalist_foreach (&db->entries, entry_remove_notifier_in_list, identifier);
+
+  notifier_destroy (identifier);
 }
 
 void
