@@ -501,8 +501,9 @@ GMZeroconfBrowser::ResolveCallback (AvahiServiceResolver *r,
      * in the contact list */
     if (tmp_list && tmp_list->data) {
       
-      contacts = g_slist_remove (contacts, GM_CONTACT (tmp_list->data));
+      contacts = g_slist_remove_link (contacts, tmp_list);
       gmcontact_delete (GM_CONTACT (tmp_list->data));
+      g_slist_free_1 (tmp_list);
     }
     
     contact->state = CONTACT_AVAILABLE;
@@ -583,13 +584,13 @@ GMZeroconfBrowser::BrowseCallback (AvahiServiceBrowser *b,
 
       contact = GM_CONTACT (l->data);
       
-      if (contact && contact->fullname) {
+      if (contact && contact->fullname && name) {
 
 	if (!strcmp (contact->fullname, name)) {
 
-	  contacts = g_slist_remove (contacts, 
-				     (gpointer) contact);
-	  gmcontact_delete (contact);
+          contacts = g_slist_remove_link (contacts, l);
+          gmcontact_delete (GM_CONTACT (l->data));
+          g_slist_free_1 (l);
 	}
       }
     }
