@@ -1,5 +1,5 @@
 
-/* GnomeMeeting -- A Video-Conferencing application
+/* Ekiga -- A VoIP and Video-Conferencing application
  * Copyright (C) 2000-2006 Damien Sandras
  *
  * This program is free software; you can redistribute it and/or modify
@@ -13,16 +13,16 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  *
- * GnomeMeting is licensed under the GPL license and as a special exception,
+ * Ekiga is licensed under the GPL license and as a special exception,
  * you have permission to link or otherwise combine this program with the
- * programs OpenH323 and Pwlib, and distribute the combination, without
- * applying the requirements of the GNU GPL to the OpenH323 program, as long
- * as you do follow the requirements of the GNU GPL for all the rest of the
- * software thus combined.
+ * programs OPAL, OpenH323 and PWLIB, and distribute the combination,
+ * without applying the requirements of the GNU GPL to the OPAL, OpenH323
+ * and PWLIB programs, as long as you do follow the requirements of the
+ * GNU GPL for all the rest of the software thus combined.
  */
 
 
@@ -75,6 +75,12 @@
 
 #define new PNEW
 
+extern "C" {
+  unsigned char linear2ulaw(int pcm_val);
+  int ulaw2linear(unsigned char u_val);
+};
+
+
 /* The class */
 GMManager::GMManager ()
 {
@@ -93,11 +99,7 @@ GMManager::GMManager ()
   gk = NULL;
   sc = NULL;
 
-  /* Use IPv6 address family by default if available. */
-#ifdef P_HAS_IPV6
-  if (PIPSocket::IsIpAddressFamilyV6Supported())
-    PIPSocket::SetDefaultIpAddressFamilyV6();
-#endif
+  PIPSocket::SetDefaultIpAddressFamilyV4();
   
   audio_tester = NULL;
 
@@ -112,15 +114,16 @@ GMManager::GMManager ()
 #endif
 
   RTPTimer.SetNotifier (PCREATE_NOTIFIER (OnRTPTimeout));
+  AvgSignalTimer.SetNotifier (PCREATE_NOTIFIER (OnAvgSignalTimeout));
   GatewayIPTimer.SetNotifier (PCREATE_NOTIFIER (OnGatewayIPTimeout));
   GatewayIPTimer.RunContinuous (PTimeInterval (5));
 
+  IPChangedTimer.SetNotifier (PCREATE_NOTIFIER (OnIPChanged));
+  IPChangedTimer.RunContinuous (120000);
   NoIncomingMediaTimer.SetNotifier (PCREATE_NOTIFIER (OnNoIncomingMediaTimeout));
 
   missed_calls = 0;
 
-  OutgoingCallTimer.SetNotifier (PCREATE_NOTIFIER (OnOutgoingCall));
-
   h323EP = NULL;
   sipEP = NULL;
   pcssEP = NULL;
@@ -174,14 +177,10 @@ GMManager::SetUpCall (const PString & call_addr,
   called_address = call_addr;
   lca_access_mutex.Signal();
   
-  OutgoingCallTimer.RunContinuous (PTimeInterval (5));
   result = OpalManager::SetUpCall ("pc:*", call_addr, call_token, NULL);
 
-  if (!result) {
-
-    OutgoingCallTimer.Stop ();
+  if (!result) 
     pcssEP->PlaySoundEvent ("busy_tone_sound");
-  }
   
   return result;
 }
@@ -211,8 +210,6 @@ void GMManager::UpdateDevices ()
   device_name = gm_conf_get_string (VIDEO_DEVICES_KEY "input_device");
   gnomemeeting_threads_leave ();
   
-  gnomemeeting_sound_daemons_suspend ();
-
   /* Do not change these values during calls */
   if (GetCallingState () == GMManager::Standby) {
 
@@ -269,7 +266,8 @@ GMManager::GetAvailableAudioMediaFormats ()
       "MS-GSM",
       "SpeexNarrow-8k",
       "SpeexWide-20.6k",
-      "LPC-10",
+      "G.726-16k",
+      "G.726-32k",
       NULL,
     };
 
@@ -414,17 +412,14 @@ GMManager::GetPCSSEndpoint ()
 
 
 PString
-GMManager::GetCurrentIP (PString protocol)
+GMManager::GetCurrentAddress (PString protocol)
 {
   OpalEndPoint *ep = NULL;
 
   PIPSocket::Address ip(PIPSocket::GetDefaultIpAny());
   WORD port = 0;
 
-  if (protocol.IsEmpty ())
-    return PString ();
-      
-  ep = FindEndPoint (protocol);
+  ep = FindEndPoint (protocol.IsEmpty () ? "sip" : protocol);
   
   if (!ep)
     return PString ();
@@ -458,7 +453,7 @@ GMManager::GetURL (PString protocol)
 
   if (!account_url)
     account_url = g_strdup_printf ("%s:%s", (const char *) protocol,
-				   (const char *) GetCurrentIP (protocol));
+				   (const char *) GetCurrentAddress (protocol));
 
   url = account_url;
   g_free (account_url);
@@ -631,8 +626,10 @@ GMManager::CreateSTUNClient (BOOL display_progress,
 {
   PWaitAndSignal m(sc_mutex);
 
-  if (sc)
+  if (sc) 
     delete (sc);
+
+  SetSTUNServer (PString ());
   
   /* Be a client for the specified STUN Server */
   sc = new GMStunClient (display_progress, 
@@ -651,6 +648,8 @@ GMManager::RemoveSTUNClient ()
   if (sc) 
     delete (sc);
   
+  SetSTUNServer (PString ());
+  
   sc = NULL;
 }
 
@@ -672,8 +671,8 @@ GMManager::OnForwarded (OpalConnection &,
   gnomemeeting_threads_enter ();
   msg = g_strdup_printf (_("Forwarding call to %s"),
 			 (const char*) forward_party);
-  gm_main_window_flash_message (main_window, msg);
-  gm_history_window_insert (history_window, msg);
+  gm_main_window_flash_message (main_window, "%s", msg);
+  gm_history_window_insert (history_window, "%s", msg);
   gnomemeeting_threads_leave ();
   g_free (msg);
 
@@ -754,7 +753,7 @@ void GMManager::GetRemoteConnectionInfo (OpalConnection & connection,
   int nbr = 0;
   contacts = 
     gnomemeeting_addressbook_get_contacts (NULL, nbr, FALSE, 
-					   NULL, utf8_url, NULL, NULL);
+					   NULL, utf8_url, NULL, NULL, NULL);
   if (contacts) {
     
     contact = GM_CONTACT (contacts->data);
@@ -826,9 +825,9 @@ GMManager::OnIncomingConnection (OpalConnection &connection,
   /* Update the log and status bar */
   msg = g_strdup_printf (_("Call from %s"), (const char *) utf8_name);
   gnomemeeting_threads_enter ();
-  gm_main_window_flash_message (main_window, msg);
-  gm_chat_window_push_info_message (chat_window, NULL, msg);
-  gm_history_window_insert (history_window, msg);
+  gm_main_window_flash_message (main_window, "%s", msg);
+  gm_chat_window_push_info_message (chat_window, NULL, "%s", msg);
+  gm_history_window_insert (history_window, "%s", msg);
   gnomemeeting_threads_leave ();
   g_free (msg);
 
@@ -866,16 +865,21 @@ GMManager::OnIncomingConnection (OpalConnection &connection,
   /* Display the action message */
   gnomemeeting_threads_enter ();
   if (short_reason) 
-    gm_main_window_flash_message (main_window, short_reason);
+    gm_main_window_flash_message (main_window, "%s", short_reason);
   if (long_reason)
-    gm_history_window_insert (history_window, long_reason);
+    gm_history_window_insert (history_window, "%s", long_reason);
   gnomemeeting_threads_leave ();
   
-  /* Update the current state and show popup if action is 1 */
-  if (reason == 0) {
-
+  /* Update the current state if action is 0 or 4.
+   * Show popup if action is 1 (show popup)
+   */
+  if (reason == 0 || reason == 4) {
+    
     SetCallingState (GMManager::Called);
     SetCurrentCallToken (connection.GetCall ().GetToken ());
+  }
+
+  if (reason == 0) {
 
     /* Update the UI */
     gnomemeeting_threads_enter ();
@@ -941,11 +945,9 @@ GMManager::OnEstablishedCall (OpalCall &call)
   forward_on_busy = gm_conf_get_bool (CALL_FORWARDING_KEY "forward_on_busy");
   gnomemeeting_threads_leave ();
   
-  /* Start refreshing the stats */
-  RTPTimer.RunContinuous (PTimeInterval (0, 1));
-
-  /* Stop the Timers */
-  OutgoingCallTimer.Stop ();
+  /* Update the timers */
+  RTPTimer.RunContinuous (PTimeInterval (1000));
+  AvgSignalTimer.RunContinuous (PTimeInterval (50));
 
   /* Update internal state */
   SetCallingState (GMManager::Connected);
@@ -955,9 +957,7 @@ GMManager::OnEstablishedCall (OpalCall &call)
   gnomemeeting_threads_enter ();
   if (called_address.IsEmpty ()) 
     gm_main_window_set_call_url (main_window, GMURL ().GetDefaultURL ());
-  gm_main_window_update_calling_state (main_window, GMManager::Connected);
   gm_main_window_set_stay_on_top (main_window, stay_on_top);
-  gm_main_window_update_calling_state (main_window, GMManager::Connected);
   gm_statusicon_update_full (statusicon, GMManager::Connected,
 			     icm, forward_on_busy);
 #ifdef HAS_DBUS
@@ -1013,8 +1013,9 @@ GMManager::OnEstablished (OpalConnection &connection)
 			      utf8_name, utf8_app);
   msg = g_strdup_printf (_("Connected with %s"), utf8_name);
   gm_main_window_set_status (main_window, msg);
-  gm_main_window_flash_message (main_window, msg);
-  gm_chat_window_push_info_message (chat_window, NULL, msg);
+  gm_main_window_flash_message (main_window, "%s", msg);
+  gm_chat_window_push_info_message (chat_window, NULL, "%s", msg);
+  gm_main_window_update_calling_state (main_window, GMManager::Connected);
   gm_chat_window_update_calling_state (chat_window, 
 				       utf8_name,
 				       utf8_url, 
@@ -1032,10 +1033,16 @@ GMManager::OnEstablished (OpalConnection &connection)
     connection.GetSession (OpalMediaFormat::DefaultAudioSessionID);
   video_session = 
     connection.GetSession (OpalMediaFormat::DefaultVideoSessionID);
-  if (audio_session)
+  if (audio_session) {
+    
     audio_session->SetIgnoreOtherSources (TRUE);
-  if (video_session)
+    audio_session->SetIgnorePayloadTypeChanges (TRUE);
+  }
+  if (video_session) {
+
     video_session->SetIgnoreOtherSources (TRUE);
+    video_session->SetIgnorePayloadTypeChanges (TRUE);
+  }
   
   if (!connection.IsOriginating ()) {
     
@@ -1070,7 +1077,6 @@ GMManager::OnClearedCall (OpalCall & call)
   BOOL forward_on_busy = FALSE;
   IncomingCallMode icm = AVAILABLE;
   ViewMode m = SOFTPHONE;
-  PString old_token;
 
   main_window = GnomeMeeting::Process ()->GetMainWindow ();
   chat_window = GnomeMeeting::Process ()->GetChatWindow ();
@@ -1079,8 +1085,10 @@ GMManager::OnClearedCall (OpalCall & call)
   dbus_component = GnomeMeeting::Process ()->GetDbusComponent ();
 #endif
   
-  if (GetCurrentCallToken() != call.GetToken())
+  if (GetCurrentCallToken() != PString::Empty() 
+      && GetCurrentCallToken () != call.GetToken())
     return;
+
   
   /* Get the config settings */
   gnomemeeting_threads_enter ();
@@ -1092,17 +1100,16 @@ GMManager::OnClearedCall (OpalCall & call)
   gnomemeeting_threads_leave ();
   
   /* Stop the Timers */
-  OutgoingCallTimer.Stop ();
   NoIncomingMediaTimer.Stop ();
   
-  gnomemeeting_sound_daemons_resume (); 
-
   /* we reset the no-data detection */
   RTPTimer.Stop ();
+  AvgSignalTimer.Stop ();
   stats.Reset ();
 
   /* Play busy tone */
-  pcssEP->PlaySoundEvent ("busy_tone_sound"); 
+  if (GetCallingState () == GMManager::Connected)
+    pcssEP->PlaySoundEvent ("busy_tone_sound"); 
 
   /* Update the various parts of the GUI */
   gnomemeeting_threads_enter ();
@@ -1112,9 +1119,6 @@ GMManager::OnClearedCall (OpalCall & call)
 				       NULL, 
 				       NULL, 
 				       GMManager::Standby);
-  gm_main_window_push_message (main_window, 
-			       GetMissedCallsNumber (), 
-			       GetMWI ());
   gm_statusicon_update_full (statusicon, GMManager::Standby,
 			     icm, forward_on_busy);
   gm_main_window_set_status (main_window, _("Standby"));
@@ -1125,16 +1129,22 @@ GMManager::OnClearedCall (OpalCall & call)
   gm_main_window_clear_signal_levels (main_window);
   gm_main_window_set_view_mode (main_window, m);
 #ifdef HAS_DBUS
-  gnomemeeting_dbus_component_set_call_state (dbus_component, old_token,
+  gnomemeeting_dbus_component_set_call_state (dbus_component,
+					      GetCurrentCallToken (),
 					      GMManager::Standby);
 #endif
+  gm_main_window_push_message (main_window, 
+			       GetMissedCallsNumber (), 
+			       GetMWI ());
   gnomemeeting_threads_leave ();
 
   /* Update internal state */
   SetCallingState (GMManager::Standby);
-  old_token = GetCurrentCallToken ();
   SetCurrentCallToken ("");
 
+  /* Reinitialize codecs */
+  re_audio_codec = tr_audio_codec = re_video_codec = tr_video_codec = "";
+
   /* Try to update the devices use if some settings were changed 
      during the call */
   UpdateDevices ();
@@ -1162,13 +1172,13 @@ GMManager::OnReleased (OpalConnection & connection)
   history_window = GnomeMeeting::Process ()->GetHistoryWindow ();
   
   /* Do nothing for the PCSS connection */
-  if (PIsDescendant(&connection, OpalPCSSConnection) || GetCallingState () == GMManager::Standby) {
+  if (PIsDescendant(&connection, OpalPCSSConnection)) {
     
     PTRACE (3, "GMManager\t Will release the connection");
     OpalManager::OnReleased (connection);
     return;
   }
-  
+
   /* Start time */
   if (connection.GetConnectionStartTime ().IsValid ())
     t = PTime () - connection.GetConnectionStartTime();
@@ -1232,17 +1242,13 @@ GMManager::OnReleased (OpalConnection & connection)
   case OpalConnection::EndedByRemoteCongestion :
     msg_reason = g_strdup (_("Congested link to remote party"));
     break;
-  case OpalConnection::EndedByUnreachable :
-    msg_reason = g_strdup (_("Remote user is unreachable"));
-    break;
-  case OpalConnection::EndedByNoEndPoint :
-    msg_reason = g_strdup (_("Remote user is unreachable"));
-    break;
   case OpalConnection::EndedByHostOffline :
     msg_reason = g_strdup (_("Remote host is offline"));
     break;
   case OpalConnection::EndedByTemporaryFailure :
-    msg_reason = g_strdup (_("Remote user is offline"));
+  case OpalConnection::EndedByUnreachable :
+  case OpalConnection::EndedByNoEndPoint :
+    msg_reason = g_strdup (_("Remote user is unreachable"));
     break;
 
   default :
@@ -1267,10 +1273,6 @@ GMManager::OnReleased (OpalConnection & connection)
     mc_access_mutex.Wait ();
     missed_calls++;
     mc_access_mutex.Signal ();
-
-    gm_main_window_push_message (main_window, 
-				 GetMissedCallsNumber (), 
-				 GetMWI ());
   }
   else
     if (!connection.IsOriginating ())
@@ -1287,9 +1289,9 @@ GMManager::OnReleased (OpalConnection & connection)
 				 t.AsString (0),
 				 msg_reason,
 				 utf8_app);
-  gm_history_window_insert (history_window, msg_reason);
-  gm_main_window_flash_message (main_window, msg_reason);
-  gm_chat_window_push_info_message (chat_window, NULL, "");
+  gm_history_window_insert (history_window, "%s", msg_reason);
+  gm_main_window_flash_message (main_window, "%s", msg_reason);
+  gm_chat_window_push_info_message (chat_window, NULL, "%s", "");
   gnomemeeting_threads_leave ();
 
   g_free (utf8_app);
@@ -1297,9 +1299,6 @@ GMManager::OnReleased (OpalConnection & connection)
   g_free (utf8_url);  
   g_free (msg_reason);
 
-  /* Reinitialize codecs */
-  re_audio_codec = tr_audio_codec = re_video_codec = tr_video_codec = "";
-
   PTRACE (3, "GMManager\t Will release the connection");
   OpalManager::OnReleased (connection);
 }
@@ -1438,22 +1437,25 @@ GMManager::SetPorts ()
   if (tcp_couple && tcp_couple [0] && tcp_couple [1]) {
 
     SetTCPPorts (atoi (tcp_couple [0]), atoi (tcp_couple [1]));
-    PTRACE (1, "Set TCP port range to " << atoi (tcp_couple [0])
-	    << atoi (tcp_couple [1]));
+    PTRACE (1, "Set TCP port range to " << setfill (':') 
+            << atoi (tcp_couple [0]) << ":"
+	    << atoi (tcp_couple [1])) << setfill (' ');
   }
 
   if (rtp_couple && rtp_couple [0] && rtp_couple [1]) {
 
     SetRtpIpPorts (atoi (rtp_couple [0]), atoi (rtp_couple [1]));
-    PTRACE (1, "Set RTP port range to " << atoi (rtp_couple [0])
-	    << atoi (rtp_couple [1]));
+    PTRACE (1, "Set RTP port range to " << setfill (':')
+            << atoi (rtp_couple [0]) << ":"
+	    << atoi (rtp_couple [1])) << setfill (' ');
   }
 
   if (udp_couple && udp_couple [0] && udp_couple [1]) {
 
     SetUDPPorts (atoi (udp_couple [0]), atoi (udp_couple [1]));
-    PTRACE (1, "Set UDP port range to " << atoi (udp_couple [0])
-	    << atoi (udp_couple [1]));
+    PTRACE (1, "Set UDP port range to " << setfill (':')
+            << atoi (udp_couple [0]) << ":"
+	    << atoi (udp_couple [1])) << setfill (' ');
   }
 
   g_free (tcp_port_range);
@@ -1579,27 +1581,34 @@ void
 GMManager::StartListeners ()
 {
   GtkWidget *main_window = NULL;
+  GtkWidget *druid = NULL;
+
+  GtkWidget *dialog = NULL;
 
   gchar *iface = NULL;
   WORD port = 0;
 
   main_window = GnomeMeeting::Process ()->GetMainWindow ();
-  
+  druid = GnomeMeeting::Process ()->GetDruidWindow ();
+
   gnomemeeting_threads_enter ();
   iface = gm_conf_get_string (PROTOCOLS_KEY "interface");
   gnomemeeting_threads_leave ();
   
-
+  /* Create the H.323 and SIP listeners */
   if (h323EP) {
   
     gnomemeeting_threads_enter ();
     port = gm_conf_get_int (H323_KEY "listen_port");
     gnomemeeting_threads_leave ();
     
+    h323EP->RemoveListener (NULL);
     if (!h323EP->StartListener (iface, port)) {
 
       gnomemeeting_threads_enter ();
-      gnomemeeting_error_dialog (GTK_WINDOW (main_window), _("Error while starting the listener for the H.323 protocol"), _("You will not be able to receive incoming H.323 calls. Please check that no other program is already running on the port used by Ekiga."));
+      dialog = gnomemeeting_error_dialog (GTK_WINDOW (main_window), _("Error while starting the listener for the H.323 protocol"), _("You will not be able to receive incoming H.323 calls. Please check that no other program is already running on the port used by Ekiga."));
+      if (gtk_window_is_active (GTK_WINDOW (druid)))
+	gtk_widget_set_parent (dialog, druid);
       gnomemeeting_threads_leave ();
     }
   }
@@ -1610,10 +1619,84 @@ GMManager::StartListeners ()
     port = gm_conf_get_int (SIP_KEY "listen_port");
     gnomemeeting_threads_leave ();
     
+    sipEP->RemoveListener (NULL);
     if (!sipEP->StartListener (iface, port)) {
       
       gnomemeeting_threads_enter ();
-      gnomemeeting_error_dialog (GTK_WINDOW (main_window), _("Error while starting the listener for the SIP protocol"), _("You will not be able to receive incoming SIP calls. Please check that no other program is already running on the port used by Ekiga."));
+      dialog = gnomemeeting_error_dialog (GTK_WINDOW (main_window), _("Error while starting the listener for the SIP protocol"), _("You will not be able to receive incoming SIP calls. Please check that no other program is already running on the port used by Ekiga."));
+      if (gtk_window_is_active (GTK_WINDOW (druid)))
+	gtk_widget_set_parent (dialog, druid);
+      gnomemeeting_threads_leave ();
+    }
+  }
+
+  g_free (iface);
+}
+
+
+void
+GMManager::OnIPChanged (PTimer &,
+			INT)
+{
+  PIPSocket::InterfaceTable ifaces;
+  OpalTransportAddress current_address;
+  PIPSocket::Address current_ip;
+  
+  PString ip;
+  PINDEX i = 0;
+  
+  BOOL found_ip = FALSE;
+  
+  gchar *iface = NULL;
+
+  gnomemeeting_threads_enter ();
+  iface = gm_conf_get_string (PROTOCOLS_KEY "interface");
+  gnomemeeting_threads_leave ();
+
+  /* Detect the valid interfaces */
+  PIPSocket::GetInterfaceTable (ifaces);
+  if (ifaces.GetSize () <= 0) {
+  
+    g_free (iface);
+    return;
+  }
+  
+  /* Is there a listener? */
+  if (sipEP->GetListeners ().GetSize () > 0) 
+    current_address = sipEP->GetListeners ()[0].GetLocalAddress ();
+  else if (h323EP->GetListeners ().GetSize () > 0) 
+    current_address = h323EP->GetListeners ()[0].GetLocalAddress ();
+  else {
+
+    g_free (iface);
+    return;
+  }
+    
+  /* Are we listening on the correct interface? */
+  if (current_address.GetIpAddress (current_ip)) {
+
+    while (i < ifaces.GetSize () && !found_ip) {
+
+      ip = ifaces [i].GetAddress ().AsString ();
+
+      if (ip == current_ip.AsString ())
+	found_ip = TRUE;
+
+      i++;
+    }
+  }
+
+  if (!found_ip) {
+
+    PTRACE (4, "Ekiga\tIP Address changed, updating listeners.");
+
+    if (ifaces.GetSize () > 1) {
+      
+      /* This will update the UI and the listeners through a GConf
+       * Notifier.
+       */
+      gnomemeeting_threads_enter ();
+      GnomeMeeting::Process ()->DetectInterfaces ();
       gnomemeeting_threads_leave ();
     }
   }
@@ -1716,9 +1799,17 @@ BOOL
 GMManager::OnOpenMediaStream (OpalConnection & connection,
 			       OpalMediaStream & stream)
 {
+  GtkWidget *main_window = NULL;
+  
   if (!OpalManager::OnOpenMediaStream (connection, stream))
     return FALSE;
 
+  main_window = GnomeMeeting::Process ()->GetMainWindow ();
+
+  gnomemeeting_threads_enter ();
+  gm_main_window_update_calling_state (main_window, GMManager::Connected);
+  gnomemeeting_threads_leave ();
+
   if (pcssEP->GetMediaFormats ().FindFormat(stream.GetMediaFormat()) == P_MAX_INDEX)
     OnMediaStream (stream, FALSE);
 
@@ -1774,9 +1865,9 @@ GMManager::OnMediaStream (OpalMediaStream & stream,
     if (!is_closing) {
       
       if (!is_video)
-	tr_audio_codec = g_strdup (codec_name);
+	tr_audio_codec = codec_name;
       else
-	tr_video_codec = g_strdup (codec_name);
+	tr_video_codec = codec_name;
       
       msg = 
 	g_strdup_printf (_("Opened codec %s for transmission"),
@@ -1799,9 +1890,9 @@ GMManager::OnMediaStream (OpalMediaStream & stream,
     if (!is_closing) {
      
       if (!is_video)
-	re_audio_codec = g_strdup (codec_name);
+	re_audio_codec = codec_name;
       else
-	re_video_codec = g_strdup (codec_name);
+	re_video_codec = codec_name;
 
       msg = 
 	g_strdup_printf (_("Opened codec %s for reception"),
@@ -1822,7 +1913,7 @@ GMManager::OnMediaStream (OpalMediaStream & stream,
 
   /* Update the GUI and menus wrt opened channels */
   gnomemeeting_threads_enter ();
-  gm_history_window_insert (history_window, msg);
+  gm_history_window_insert (history_window, "%s", msg);
   gm_main_window_update_sensitivity (main_window, is_video, is_video?is_receiving_video:is_receiving_audio, is_video?is_transmitting_video:is_transmitting_audio);
   gm_main_window_set_channel_pause (main_window, FALSE, is_video);
   gm_main_window_set_call_info (main_window, 
@@ -1861,10 +1952,8 @@ GMManager::UpdateRTPStats (PTime start_time,
       re_bytes = audio_session->GetOctetsReceived ();
       tr_bytes = audio_session->GetOctetsSent ();
 
-      stats.a_re_bandwidth = (re_bytes - stats.re_a_bytes) 
-	/ (1024.0 * elapsed_seconds);
-      stats.a_tr_bandwidth = (tr_bytes - stats.tr_a_bytes) 
-	/ (1024.0 * elapsed_seconds);
+      stats.a_re_bandwidth = PMAX ((re_bytes - stats.re_a_bytes) / (1024.0 * elapsed_seconds), 0);
+      stats.a_tr_bandwidth = PMAX ((tr_bytes - stats.tr_a_bytes) / (1024.0 * elapsed_seconds), 0);
 
       buffer_size = audio_session->GetJitterBufferSize ();
       time_units = audio_session->GetJitterTimeUnits ();
@@ -1873,7 +1962,7 @@ GMManager::UpdateRTPStats (PTime start_time,
 
       stats.re_a_bytes = re_bytes;
       stats.tr_a_bytes = tr_bytes;
-      
+
       stats.total_packets += audio_session->GetPacketsReceived ();
       stats.lost_packets += audio_session->GetPacketsLost ();
       stats.late_packets += audio_session->GetPacketsTooLate ();
@@ -1885,10 +1974,8 @@ GMManager::UpdateRTPStats (PTime start_time,
       re_bytes = video_session->GetOctetsReceived ();
       tr_bytes = video_session->GetOctetsSent ();
 
-      stats.v_re_bandwidth = (re_bytes - stats.re_v_bytes) 
-	/ (1024.0 * elapsed_seconds);
-      stats.v_tr_bandwidth = (tr_bytes - stats.tr_v_bytes) 
-	/ (1024.0 * elapsed_seconds);
+      stats.v_re_bandwidth = PMAX ((re_bytes - stats.re_v_bytes) / (1024.0 * elapsed_seconds), 0);
+      stats.v_tr_bandwidth = PMAX ((tr_bytes - stats.tr_v_bytes) / (1024.0 * elapsed_seconds), 0);
 
       stats.re_v_bytes = re_bytes;
       stats.tr_v_bytes = tr_bytes;
@@ -1905,6 +1992,51 @@ GMManager::UpdateRTPStats (PTime start_time,
 }
 
 
+void 
+GMManager::OnAvgSignalTimeout (PTimer &,
+                               INT)
+{
+  GtkWidget *main_window = NULL;
+  
+  PSafePtr <OpalCall> call = NULL;
+  PSafePtr <OpalConnection> connection = NULL;
+
+  OpalRawMediaStream *audio_stream = NULL;
+
+  float output = 0;
+  float input = 0;
+  
+  main_window = GnomeMeeting::Process ()->GetMainWindow ();
+
+  call = FindCallWithLock (GetCurrentCallToken ());
+  if (call != NULL) {
+
+    connection = GetConnection (call, FALSE);
+
+    if (connection != NULL) {
+
+      audio_stream = 
+        (OpalRawMediaStream *) 
+        connection->GetMediaStream (OpalMediaFormat::DefaultAudioSessionID, 
+                                    FALSE);
+      if (audio_stream)
+        output = (linear2ulaw (audio_stream->GetAverageSignalLevel ()) ^ 0xff) / 100.0;
+
+      audio_stream = 
+        (OpalRawMediaStream *) 
+        connection->GetMediaStream (OpalMediaFormat::DefaultAudioSessionID, 
+                                    TRUE);
+      if (audio_stream)
+        input = (linear2ulaw (audio_stream->GetAverageSignalLevel ()) ^ 0xff) / 100.0;
+    } 
+
+    gdk_threads_enter ();
+    gm_main_window_set_signal_levels (main_window, output, input);
+    gdk_threads_leave ();
+  }
+}
+
+
 void 
 GMManager::OnRTPTimeout (PTimer &, 
 			  INT)
@@ -1982,7 +2114,7 @@ GMManager::OnRTPTimeout (PTimer &,
 
 
   gdk_threads_enter ();
-  gm_main_window_push_info_message (main_window, msg);
+  gm_main_window_push_info_message (main_window, "%s", msg);
   gm_main_window_update_stats (main_window,
 			       lost_packets_per,
 			       late_packets_per,
@@ -2003,7 +2135,7 @@ void
 GMManager::OnGatewayIPTimeout (PTimer &,
 				    INT)
 {
-  PHTTPClient web_client ("GnomeMeeting");
+  PHTTPClient web_client ("Ekiga");
   PString html, ip_address;
   gboolean ip_checking = false;
 
@@ -2041,17 +2173,6 @@ GMManager::OnGatewayIPTimeout (PTimer &,
 }
 
 
-void
-GMManager::OnOutgoingCall (PTimer &,
-			    INT) 
-{
-  pcssEP->PlaySoundEvent ("ring_tone_sound");
-
-  if (OutgoingCallTimer.IsRunning ())
-    OutgoingCallTimer.RunContinuous (PTimeInterval (0, 3));
-}
-
-
 BOOL 
 GMManager::DeviceVolume (PSoundChannel *sound_channel,
 			  BOOL is_encoding,
@@ -2219,13 +2340,25 @@ GMManager::SetCallOnHold (PString callToken,
 {
   PSafePtr<OpalCall> call = FindCallWithLock (callToken);
   OpalConnection *connection = NULL;
-  
+#ifdef HAS_DBUS
+  GObject *dbus_component = NULL;
+#endif  
+
+#ifdef HAS_DBUS
+  dbus_component = GnomeMeeting::Process ()->GetDbusComponent ();
+#endif
+
   if (call != NULL) {
 
     connection = GetConnection (call, TRUE);
 
     if (connection != NULL) {
 
+#ifdef HAS_DBUS
+      gnomemeeting_dbus_component_set_call_on_hold (dbus_component,
+						    callToken,
+						    state);
+#endif
       if (state) 
 	connection->HoldConnection ();
       else 
@@ -2251,7 +2384,7 @@ GMManager::SendDTMF (PString callToken,
     connection = GetConnection (call, TRUE);
 
     if (connection != NULL) 
-      connection->SendUserInputTone(dtmf [0], 50);
+      connection->SendUserInputTone(dtmf [0], 0);
   }
 }
 
