@@ -374,14 +374,16 @@ gm_aw_edit_account_dialog_run (GtkWidget *accounts_window,
   GtkWidget *table = NULL;
   GtkWidget *label = NULL;
 
-  PRegularExpression regex ("^[a-z0-9][-._a-z0-9@ ]*$", 
-			    PRegularExpression::IgnoreCase);
-
   /* FIXME: that catches the most common allowed characters only, for anything else,
    * we will need a function to encode to % HEX HEX format for use in an URL or similar
    */
+
+  /* this one breaks for damien (23 Mar 07) */
+  /*  PRegularExpression regex_username ("^[a-z0-9!#$%&*'*+-/=?^_`{}~][@.a-z0-9!#$%&*'*+-/=?^_`{}~]*$",
+		                     PRegularExpression::IgnoreCase); */
+  /* the "old one" */
   PRegularExpression regex_username ("^[-+._a-z0-9@ ][-._a-z0-9#@* ]*$",
-		                     PRegularExpression::IgnoreCase);
+				     PRegularExpression::IgnoreCase);
 
   PString username;
   PString auth_username;
@@ -620,10 +622,34 @@ gm_aw_edit_account_dialog_run (GtkWidget *accounts_window,
       /* Check at least an account name, registrar, 
        * and username are provided */
       if (protocol == 0) // SIP
-	valid = (username.FindRegEx (regex_username) != P_MAX_INDEX
-		 && account_name.FindRegEx (regex) != P_MAX_INDEX);
+       {
+	 valid = TRUE;
+	 if (username.FindRegEx (regex_username) == P_MAX_INDEX)
+	  {
+	    /* FIXME change strings after stringfreeze to something more useful - js */
+	    gnomemeeting_error_dialog (GTK_WINDOW (dialog), _("Missing information"),
+				       _("Please make sure to provide a valid account name, host name, and user name."));
+	    valid = FALSE;
+	  }
+	 if (account_name.Find ('|') != P_MAX_INDEX)
+	  {
+	    /* FIXME change strings after stringfreeze to something more useful - js */
+	    gnomemeeting_error_dialog (GTK_WINDOW (dialog), _("Missing information"), 
+				       _("Please make sure to provide a valid account name, host name, and user name."));
+	    valid = FALSE;
+	  }
+       }
       else // H323
-	valid = (account_name.FindRegEx (regex) != P_MAX_INDEX);
+       {
+	 valid = TRUE;
+	 if (account_name.Find ('|') != P_MAX_INDEX)
+	  {
+	    /* FIXME change strings after stringfreeze to something more useful - js */
+	    valid = FALSE;
+	    gnomemeeting_error_dialog (GTK_WINDOW (dialog), _("Missing information"),
+				       _("Please make sure to provide a valid account name, host name and registration timeout."));
+	  }
+       }
 
       if (valid) {
 
@@ -679,12 +705,6 @@ gm_aw_edit_account_dialog_run (GtkWidget *accounts_window,
 	else 
 	  gnomemeeting_account_add (account);
       }
-      else {
-	if (protocol == 0) // SIP
-	  gnomemeeting_error_dialog (GTK_WINDOW (dialog), _("Missing information"), _("Please make sure to provide a valid account name, host name, and user name."));
-	else // H323
-	  gnomemeeting_error_dialog (GTK_WINDOW (dialog), _("Missing information"), _("Please make sure to provide a valid account name, host name and registration timeout."));
-      }
       break;
 
     case GTK_RESPONSE_DELETE_EVENT:
